//
//  PathDirectionHelperMeethods.swift
//  PointrKitUI-App
//
//  Created by test on 28/10/21.
//

import Foundation
import PointrKit

class PathDirectionHelperMethods {

    /**
     * The following three methods [getDirectionMessageFromDirectionType] and [getMessageForPortal]
     * and [getNodeMessage] are used to generate a meaningful & translatable message from a PathDirection.
     * The  strings in these methods are given for demonstrative purposes, the implementor will need to
     * create relevant localised values in their project.
     */
    // region message
    func getDirectionMessageFromDirectionType(direction: PTRPathDirection, nextDirection: PTRPathDirection?) -> String? {
        let directionType = direction.type
        switch (directionType) {
            case .turnLeft:
                return "Turn Left"
            case .invalid:
                return "Invalid"
            case .turnSlightlyLeft:
                return "Turn slightly left"
            case .turnSlightlyRight:
                return "Turn slightly right"
            case .turnRight:
                return "Turn right"
            case .rotateBack:
                return "Rotate back"
            case .rotateRight:
                return "Rotate right"
            case .rotateLeft:
                return "Rotate left"
            case .sameLevelPortal:
                return "Same level portal"
            case .upwardPortal:
                return "Upward portal"
            case .downwardPortal:
                return "Downward portal"
            case .interfacilityPortal:
                return "Inter facility portal"
            case .destination:
                return "Destination"
            @unknown default:
                return ""
        }
    }
    
    func getMessageForPortal(directionType: PTRPathDirectionType,
                             direction: PTRPathDirection,
                             nextDirection: PTRPathDirection?) -> String {
        var nextLevelTitleNullable = nextDirection?.location.level?.title
        
        var nextLevelTitle = nextLevelTitleNullable == nil ?
                             "level \(nextDirection?.location.level?.index)" :
                             nextLevelTitleNullable

        var nextFacilityNullable = nextDirection?.location.facility
        var nextFacilityTitle = ""
        
        if (nextFacilityNullable?.internalIdentifier == 1) {
            nextFacilityTitle = "facility title of facility with id 1"
        } else if (nextFacilityNullable?.internalIdentifier == 2) {
            nextFacilityTitle = "facility title of facility with id 2"
        } else {
            nextFacilityTitle = "facility title of facility with id 3"
        }

        var portalName = getNodeMessage(direction.nodeType)

        if directionType == .upwardPortal {
            return "Example \(portalName) \(nextLevelTitle)"
        } else if directionType == .upwardPortal {
            return "Example \(portalName) \(nextLevelTitle)"
        } else if directionType == .upwardPortal {
            return "Example \(portalName) \(nextFacilityTitle)"
        } else {
            return ""
        }
    }

    func getMessageForPortal(directionType: PTRPathDirectionType,
                             direction: PTRPathDirection,
                             nextDirection: PTRPathDirection?) -> String {
        
        var nextLevelTitleNullable = nextDirection?.location.level?.title
        var nextLevelTitle = nextLevelTitleNullable == nil ?
                             "level \(nextDirection?.location.level?.index)" :
                             nextLevelTitleNullable

        var nextFacilityNullable = nextDirection?.location.facility
        var nextFacilityTitle = ""
        
        if (nextFacilityNullable?.internalIdentifier == 1) {
            nextFacilityTitle = "facility title of facility with id 1"
        } else if (nextFacilityNullable?.internalIdentifier == 2) {
            nextFacilityTitle = "facility title of facility with id 2"
        } else {
            nextFacilityTitle = "facility title of facility with id 3"
        }

        var portalName = getNodeMessage(direction.nodeType)

        if directionType == .upwardPortal {
            return "Example \(portalName) \(nextLevelTitle)"
        } else if directionType == .upwardPortal {
            return "Example \(portalName) \(nextLevelTitle)"
        } else if directionType == .upwardPortal {
            return "Example \(portalName) \(nextFacilityTitle)"
        } else {
            return ""
        }
    }

    func getNodeMessage(nodeType: PTRNodeType) -> String {
        switch (nodeType) {
            
        case .unknown:
            return "Unknown"
        case .regular:
            return "Regular"
        case .rotation:
            return "Rotation"
        case .turn:
            return "Turn"
        case .lift:
            return "Lift"
        case .escalator:
            return "Escalator"
        case .travelator:
            return "Travelator"
        case .angularStairs:
            return "Angular stairs"
        case .rotationalStairs:
            return "Rotational stairs"
        case .serviceLift:
            return "Service lift"
        case .skewedLift:
            return "Skewed lift"
        case .securityControl:
            return "Security control"
        case .borderControl:
            return "Border control"
        case .ramp:
            return "Ramp"
        case .steps:
            return "Steps"
        case .serviceStairs:
            return "Service stairs"
        case .train:
            return "Train"
        case .shuttle:
            return "Shuttle"
        case .metro:
            return "Metro"
        case .bridge:
            return "Bridge"
        @unknown default:
            return ""
        }
    }
    // endregion

    /**
     * The following method [getImageResOfDirection] method can be used to supply an image for
     * the direction depending on its PTRPathDirectionType and PTRNodeType. The images
     * returned here are for demonstrative purposes, please replace them with actual images
     * in your project bundle. Or else the project will not compile once you copy this code.
     */
    // region Image of direction
    func getImageResOfDirection(directionType: PTRPathDirectionType,
                                nodeType: PTRNodeType?) -> UIImage {
        
        switch (directionType) {
            
        case .invalid:
            return UIImage(named: "invalid")!
        case .turnSlightlyLeft:
            return UIImage(named: "turnSlightlyLeft")!
        case .turnLeft:
            return UIImage(named: "turnLeft")!
        case .turnSlightlyRight:
            return UIImage(named: "turnSlightlyRight")!
        case .turnRight:
            return UIImage(named: "turnRight")!
        case .rotateBack:
            return UIImage(named: "rotateBack")!
        case .rotateRight:
            return UIImage(named: "rotateRight")!
        case .rotateLeft:
            return UIImage(named: "rotateLeft")!
        case .sameLevelPortal:
            return UIImage(named: "sameLevelPortal")!
        case .upwardPortal:
            if (nodeType == .angularStairs || nodeType == .rotationalStairs || nodeType == .serviceStairs) {
                return UIImage(named: "stairs")!
            } else if (nodeType == .lift || nodeType == .serviceLift || nodeType == .skewedLift) {
                return UIImage(named: "elevator")!
            } else if (nodeType == .escalator) {
                return UIImage(named: "escalator")!
            } else if (nodeType == .ramp) {
                return UIImage(named: "ramp")!
            } else {
                return UIImage(named: "escalator")!
            }
        case .downwardPortal:
            if (nodeType == .angularStairs || nodeType == .rotationalStairs || nodeType == .serviceStairs) {
                return UIImage(named: "stairs")!
            } else if (nodeType == .lift || nodeType == .serviceLift || nodeType == .skewedLift) {
                return UIImage(named: "elevator")!
            } else if (nodeType == .escalator) {
                return UIImage(named: "escalator")!
            } else if (nodeType == .ramp) {
                return UIImage(named: "ramp")!
            } else {
                return UIImage(named: "escalator")!
            }
        case .interfacilityPortal:
            return UIImage(named: "interfacilityPortal")!
        case .destination:
            return UIImage(named: "destination")!
        @unknown default:
            return UIImage()
        }
    }
    //endregion
}
